{% extends base_template|default:"base.html" %}

{% load i18n modelview_detail %}

{% block title %}{{ object }} - {{ verbose_name }} - {{ block.super }}{% endblock %}

{% block content %}
<h2>{{ object }}</h2>

<table>
{% for title, value in object|model_details %}
    <tr class="{% cycle "odd" "even" %}">
        <th>{{ title|capfirst }}</th>
        <td>{{ value }}</td>
    </tr>
{% endfor %}
</table>

<div class="buttons">
    <a href="delete/" class="button"><span>{% trans "delete"|capfirst %}</span></a>
    <a href="edit/"class="button"><span>{% trans "edit"|capfirst %}</span></a>
</div>

{% endblock %}
